from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By

from Samuranium.Config import Config
from Samuranium.WebElement import WebElement
from selenium import webdriver
from Samuranium.utils.SelectorStrategies import SelectorStrategies


class WebElements:
    def __init__(self, samuranium_instance, selector):
        self.config = Config()
        self.samuranium_instance = samuranium_instance
        self.driver: webdriver = samuranium_instance.driver
        self.selector_strategies = SelectorStrategies(selector)
        self.logger = samuranium_instance.logger
        self.selector = selector
        self.elements = self.find_elements()

    def __len__(self):
        return len(self.elements)

    def find_elements(self):
        elements = []
        for strategy_name, method in self.selector_strategies.finder_strategies().items():
            if method == By.XPATH:
                elements = self.__find_all_by_xpath()
                if len(elements) > 0:
                    break
            elements = self.driver.find_elements(method, self.selector)
            if len(elements) > 0:
                break
        if elements:
            return [WebElement(samuranium_instance=self.samuranium_instance,
                               selector=self.selector,
                               element=element) for element in elements]

    def __find_all_by_xpath(self):
        for finder_name, xpath_strategy in self.selector_strategies.xpath_strategies().items():
            try:
                elements = self.driver.find_elements_by_xpath(xpath_strategy.format(self.selector))
                if elements:
                    return elements
            except NoSuchElementException:
                pass
        return []




import os

PROJECT_ROOT_PATH = os.getcwd()
REPORT_PATH = os.path.join(PROJECT_ROOT_PATH, 'reports')
REPORT_SCREENSHOTS_PATH = os.path.join(REPORT_PATH, 'screenshots')
OUTPUT_PATH = os.path.join(PROJECT_ROOT_PATH, 'output')
SCREENSHOTS_PATH = os.path.join(OUTPUT_PATH, 'screenshots')



def get_class_variables_values(class_obj):
    """Gets a list of values for every class variable"""
    class_variable_keys = [i for i in class_obj.__dict__.keys() if i[:1] != '_']
    return [class_obj.__dict__.get(key) for key in class_variable_keys]


def get_class_variables_dict(class_obj):
    """returns a dict containing class variables and values. __ methods and variables
    are excluded """
    return dict((key, value) for (key, value) in class_obj.__dict__.items() if key[:1] != '_')



import functools
import time


def get_current_time():
    return time.perf_counter()


def timer(func):
    """Print the runtime of the decorated function"""

    @functools.wraps(func)
    def wrapper_timer(*args, **kwargs):
        start_time = get_current_time()
        value = func(*args, *kwargs)
        end_time = get_current_time()
        run_time = end_time - start_time
        print(f"Finished {func.__name__!r} in {run_time:.4f} seconds")
        return value

    return wrapper_timer



class Logger:

    @staticmethod
    def log(message):
        print(f'Log - {message}')

    @staticmethod
    def error(message):
        print(f'Error - {message}')

    @staticmethod
    def debug(message):
        print(f'debug - {message}')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['Samuranium', 'Samuranium.exceptions', 'Samuranium.utils']

package_data = \
{'': ['*']}

install_requires = \
['configparser>=5.0.2,<6.0.0',
 'selenium>=3.141.0,<4.0.0',
 'webdriver-manager>=3.4.2,<4.0.0']

setup_kwargs = {
    'name': 'samuranium2',
    'version': '0.5.2',
    'description': 'Samuranium Automation Framework',
    'long_description': None,
    'author': 'Alexis Giovoglanian (malazay)',
    'author_email': 'alexisgiovoglanian@southerncode.us',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import configparser
import os

from Samuranium.utils.paths import PROJECT_ROOT_PATH


class Config:
    def __init__(self, context=None):
        self.config_file_location = os.path.join(PROJECT_ROOT_PATH, '.samuranium')
        self.config = configparser.ConfigParser()
        self.config.read(self.config_file_location)
        self.context = context

    @property
    def cli_params(self):
        if self.context:
            return self.context.config.userdata
        return []

    @property
    def is_mobile(self):
        return True

    @property
    def default_browser_options(self):
        return {
            'browser': 'CHROME',
            'headless': False
        }

    @property
    def browser(self):
        if self.cli_params:
            cli_browser = self.cli_params.get('browser', None)
            if cli_browser:
                return cli_browser
        return self.get_property('browser', 'browser')

    @property
    def browser_options(self):
        if self.config.has_section('browser'):
            return dict(self.config.items('browser'))
        return self.default_browser_options

    @property
    def default_wait_time(self):
        default_wait_time = 30
        return float(self.get_property('browser', 'max_wait_time') or default_wait_time)

    def get_property(self, category, key):
        try:
            return self.config.get(category, key).replace('"', '').replace("'", '')
        except:
            return None


import os.path
from pathlib import Path

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By

from Samuranium.utils.classes import get_class_variables_dict
from Samuranium.utils.paths import PROJECT_ROOT_PATH


class SelectorStrategies:
    def __init__(self, selector):
        self.selector = selector
        self.config_file_location = os.path.join(PROJECT_ROOT_PATH, 'selectors.json')

    def load_custom_selectors(self):
        if self.config_file_location:
            pass

    @staticmethod
    def finder_strategies():
        return get_class_variables_dict(By)

    @staticmethod
    def xpath_strategies():
        return {'match_xpath': '{}', 'exact_text': '//*[text()="{}"]',
                'normalize_text': '//*[not(self::script)][text()[normalize-space()="{}"]]',
                'contains_text': '//*[not(self::script)][contains(text(),"{}")]',
                'normalize_contains_text':
                    '//*[not(self::script)][contains(normalize-space(.), "{}")]',
                }

    @staticmethod
    def css_strategies():
        return {'match_css': '{}', 'class_name': '.{}', 'id': '#{}'}


